==> acl_2.0.8-1.tar.gz: acl-2.0.8/libacl/acl_create_entry.c <==
/*
  File: acl_create_entry.c

  Copyright (C) 1999, 2000
  Andreas Gruenbacher <a.gruenbacher@computer.org>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "libacl.h"


acl_entry_obj *
__acl_create_entry_obj(acl_obj *acl_obj_p)
{
	acl_entry_obj *entry_obj_p;

	entry_obj_p = new_obj_p(acl_entry);
	if (!entry_obj_p)
		return NULL;
	acl_obj_p->aused++;

	/* Insert at the end of the entry ring */
	entry_obj_p->eprev = acl_obj_p->aprev;
	entry_obj_p->enext = (acl_entry_obj *)acl_obj_p;
	entry_obj_p->eprev->enext = entry_obj_p;
	entry_obj_p->enext->eprev = entry_obj_p;
	
	entry_obj_p->econtainer = acl_obj_p;
	init_acl_entry_obj(*entry_obj_p);

	return entry_obj_p;
}

/* 23.4.7 */
int
acl_create_entry(acl_t *acl_p, acl_entry_t *entry_p)
{
	acl_obj *acl_obj_p;
	acl_entry_obj *entry_obj_p;
	if (!acl_p || !entry_p) {
		errno = EINVAL;
		return -1;
	}
	acl_obj_p = ext2int(acl, *acl_p);
	if (!acl_obj_p)
		return -1;
	entry_obj_p = __acl_create_entry_obj(acl_obj_p);
	if (entry_obj_p == NULL)
		return -1;
	*entry_p = int2ext(entry_obj_p);
	return 0;
}

==> acl_2.2.47-2.tar.gz: acl-2.2.47/libacl/acl_create_entry.c <==
/*
  File: acl_create_entry.c

  Copyright (C) 1999, 2000
  Andreas Gruenbacher <a.gruenbacher@bestbits.at>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "libacl.h"


acl_entry_obj *
__acl_create_entry_obj(acl_obj *acl_obj_p)
{
	acl_entry_obj *entry_obj_p;

	if (acl_obj_p->aprealloc == acl_obj_p->aprealloc_end) {
		entry_obj_p = new_obj_p(acl_entry);
		if (!entry_obj_p)
			return NULL;
	} else {
		entry_obj_p = --acl_obj_p->aprealloc_end;
		new_obj_p_here(acl_entry, entry_obj_p);
	}
	acl_obj_p->aused++;

	/* Insert at the end of the entry ring */
	entry_obj_p->eprev = acl_obj_p->aprev;
	entry_obj_p->enext = (acl_entry_obj *)acl_obj_p;
	entry_obj_p->eprev->enext = entry_obj_p;
	entry_obj_p->enext->eprev = entry_obj_p;
	
	entry_obj_p->econtainer = acl_obj_p;
	init_acl_entry_obj(*entry_obj_p);

	return entry_obj_p;
}

/* 23.4.7 */
int
acl_create_entry(acl_t *acl_p, acl_entry_t *entry_p)
{
	acl_obj *acl_obj_p;
	acl_entry_obj *entry_obj_p;
	if (!acl_p || !entry_p) {
		if (entry_p)
			*entry_p = NULL;
		errno = EINVAL;
		return -1;
	}
	acl_obj_p = ext2int(acl, *acl_p);
	if (!acl_obj_p)
		return -1;
	entry_obj_p = __acl_create_entry_obj(acl_obj_p);
	if (entry_obj_p == NULL)
		return -1;
	*entry_p = int2ext(entry_obj_p);
	return 0;
}
